package crawler;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.SocketException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import output.DataSetOutputter;
import storers.Hotel;
import storers.Review;
import storers.Service;

/**
 * Class for crawling TripAdvisor.
 *
 */
public class TripAdvisorParser {

	//Patterns for each hotel
	private static final Pattern reviewPattern = Pattern.compile("<a href=\"(.*?)\" class=\"property_title\".*?>");
	private static final Pattern numberLinksPattern = Pattern.compile("<i>.*?</i> of <b>(.*?)</b>");
	private static final Pattern searchLinkPattern = Pattern.compile("<a href=\"(.*?)\" onclick=\"setOneTimeCookie(.*?);\">");
	private static final Pattern nextReviewPattern = Pattern.compile("<a href=\"(.*?)\" class=\"guiArw sprite-pageNext");
	private static final Pattern numberReviewsPattern = Pattern.compile("<label for=\"sortOrder\">(.*?) reviews sorted by</label> <input name=\"sortOrder\" id=\"sortOrder\" type=\"hidden\"/>");
	private static final Pattern numberStarsPattern = Pattern.compile("<span class=\"rate rate_cl cl.*?alt=\"(.*?) of.*?\"");
	private static final Pattern namePattern = Pattern.compile("(.*?): Traveler Reviews </h1>");
	private static final Pattern addressPattern = Pattern.compile("property=\"v:street-address\">(.*?)</span> </span>");
	private static final Pattern latitudePattern = Pattern.compile("lat: (.*?),");
	private static final Pattern longitudePattern = Pattern.compile("lng: (.*?),");
	
	//Patterns for each review
	private static final Pattern headlinePattern = Pattern.compile("<div class=\"quote\">&#x201c;(.*?)&#x201d;</div>");
	private static final Pattern ratingPattern = Pattern.compile("<div class=\"rating reviewItemInline\"><span class=\"rate rate_s s(.*?)0\">");
	private static final Pattern datePattern = Pattern.compile("<span class=\"ratingDate\">Reviewed (.*?\\d{4})");
	private static final Pattern reviewTextPattern = Pattern.compile("<div class=\"entry\"><p id=\"review_.*?\">(.*?)</p>");
	//private static final Pattern roomTipPattern = Pattern.compile("<b>Room Tip:</b> (.*?)<div class=\"");
	private static final Pattern profilePattern = Pattern.compile("<span class=\"recommend-titleInline\">Stayed.*?, traveled (.*?)</span>");
	private static final Pattern likedPattern = Pattern.compile("<li class=\"qItem\"><b>Liked</b> &#x2014; (.*?)</li>");
	private static final Pattern dislikedPattern = Pattern.compile("<li class=\"qItem\"><b>Disliked</b> &#x2014; (.*?)</li>");
	private static final Pattern numberReviewHelpfulPattern = Pattern.compile("<div class=\"hlpNmbr\">(.*?)</div>");
	private static final Pattern numberUserReviewsPattern = Pattern.compile("<div class=\"totalText\">(.*?) review");
	private static final Pattern numberHelpfulUserPattern = Pattern.compile("/><div class=\"totalText\">(.*?) helpful votes </div>");
	private static final Pattern userNamePattern = Pattern.compile("span class=\".*?hvrIE6 mbrName_.*?\">(.*?)</span>");

	private DateTimeFormatter dtf = DateTimeFormat.forPattern("MMMM dd, yyyy").withZone(DateTimeZone.UTC);
	private String urlBasic = "http://www.tripadvisor.com";
	private int sleepTime = 1500;
	List<Service> service;
	int numberReviewsEachHotel = 0;

	/**
	 * Constructor
	 */
	public TripAdvisorParser(){
		service = new ArrayList<Service>();
	}

	/**
	 * Start crawling function for TripAdvisor
	 * @param query Input query
	 * @return List of services
	 */
	public List<Service> crawl(String[] query){
		System.out.println("Searching for hotels in " + query[0] + "...");
		URL url;
		BufferedReader in;
		String id;
		int numberHotels = 0;
		String queryModified = "";
		String link = "";
		int counter = 0;

		for (int i = 0; i < query.length; i++){
			if (i == query.length - 1){
				queryModified += query[i];
			}
			else {
				queryModified += query[i] + "+";
				queryModified = queryModified.replaceAll("\\s", "+");
			}
		}
		try {
			//Thread.sleep(sleepTime);
			url = new URL(urlBasic + "/Search?q="+queryModified);
			in = new BufferedReader(new InputStreamReader(url.openStream(), "UTF-8"));
			String inputLine;

			//Gets the first link after the searched words
			while((inputLine = in.readLine()) != null){
				inputLine = inputLine.trim();
				Matcher matchSearchLink = searchLinkPattern.matcher(inputLine);
				if (matchSearchLink.find()){
					link = matchSearchLink.group(1);
					break;
				}
			}
			String[] linkSplit = link.split("-");
			id = linkSplit[1];

			url = new URL(urlBasic+link);
			in = new BufferedReader(new InputStreamReader(url.openStream(), "UTF-8"));

			//Get number of hotels on page
			while((inputLine = in.readLine()) != null){
				inputLine = inputLine.trim();
				Matcher matchNumberLinks = numberLinksPattern.matcher(inputLine);
				if (matchNumberLinks.find()){
					numberHotels = Integer.parseInt(matchNumberLinks.group(1));
				}
			}

			System.out.println("Number of hotels: " + numberHotels);
			for (int j = 0; j < numberHotels; j += 30 ){
				String hotelLinks = linkSplit[0] + "-" + linkSplit[1] + "-" + "oa" + j + "-" + linkSplit[2] + "-" + linkSplit[3];
				url = new URL(urlBasic+hotelLinks);
				in = new BufferedReader(new InputStreamReader(url.openStream(), "UTF-8"));
				while((inputLine = in.readLine()) != null){
					inputLine = inputLine.trim();

					if (inputLine.contains("<a href=\"/Hotel_Review-" + id + "-")){
						Matcher matchReview = reviewPattern.matcher(inputLine);
						if (matchReview.find()){
							String temp = matchReview.group(1);
							temp = temp.replace("-Reviews", "");
							temp = temp.replace("Hotel_Review", "ShowUserReviews");
							DataSetOutputter outXML = new DataSetOutputter("tripadvisor", query[0]);
							outXML.readXML();
							counter++;
							System.out.println("");
							System.out.println("Hotel number: " + counter + "/" + numberHotels);
							Service h = getHotel(temp, outXML);
							if (h == null){
								continue;
							}
							service.add(h);
							outXML.output(service);
							service.remove(0);
						}
					}
				}
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (SocketException e){
			return crawl(query);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return service;
	}
	/**
	 * Get hotels
	 * @param hotelUrl Url of the hotel
	 * @return The hotel with all the reviews
	 */
	private Service getHotel(String hotelUrl, DataSetOutputter outXML){
		numberReviewsEachHotel = 0;
		String nextPage = null;
		String hotelName = "";
		String address = "";
		URL url;
		BufferedReader in;
		String inputLine = "";
		double numberStars = 0.0;
		double lat = 0.0;
		double lng = 0.0;
		Matcher matchNumberReviews;
		Matcher matchNextReview;
		Matcher matchNumberStars;
		Matcher matchName;
		Matcher matchAddress;
		Matcher matchLatitude;
		Matcher matchLongitude;

		String reviewLink = hotelUrl; 
		try {
			//Thread.sleep(sleepTime);
			url = new URL(urlBasic + reviewLink);
			in = new BufferedReader(new InputStreamReader(url.openStream(), "UTF-8"));
			while((inputLine = in.readLine()) != null){
				inputLine = inputLine.trim();
				matchNumberReviews = numberReviewsPattern.matcher(inputLine);
				if (matchNumberReviews.find()){
					String temp = matchNumberReviews.group(1);
					temp = temp.replaceAll(",", "");
					numberReviewsEachHotel = Integer.parseInt(temp);
					System.out.println("Number of hotel reviews: " + numberReviewsEachHotel);
				}
				matchNextReview = nextReviewPattern.matcher(inputLine);
				if (matchNextReview.find()){
					nextPage = matchNextReview.group(1);
					//allReviewLinks.add(matchNextReview.group(1));
				}
				matchNumberStars = numberStarsPattern.matcher(inputLine);
				if (matchNumberStars.find()){
					numberStars = Double.parseDouble(matchNumberStars.group(1));
				}
				matchName = namePattern.matcher(inputLine);
				if (matchName.find()){
					hotelName = matchName.group(1);
					System.out.println("Name of hotel: " + hotelName);
					if (outXML.serviceExists(hotelName)){
						System.out.println("Hotel exist.");
						return null;
					}
				}
				matchAddress = addressPattern.matcher(inputLine);
				if (matchAddress.find()){
					address = matchAddress.group(1);
					address = address.replaceAll("<.*?>", "");
				}
				matchLatitude = latitudePattern.matcher(inputLine);
				if (matchLatitude.find()){
					lat = Double.parseDouble(matchLatitude.group(1));
				}
				matchLongitude = longitudePattern.matcher(inputLine);
				if (matchLongitude.find()){
					lng = Double.parseDouble(matchLongitude.group(1));
				}
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (SocketException e) {
			return getHotel(hotelUrl, outXML);
		} catch (IOException e) {
			e.printStackTrace();
		} //catch (InterruptedException e) {
//			e.printStackTrace();
//		}
		Service hotel = new Hotel(hotelName, address, numberStars);
		hotel.setLatitude(lat);
		hotel.setLongitude(lng);
		hotel.setReviews(getReviews(reviewLink, nextPage));
		return hotel;
	}

	/**
	 * Get reviews
	 * @param allReviewLinks All review links
	 * @return A list of all the reviews
	 */
	public List<Review> getReviews(String currentPage, String nextPage){
		System.out.println("Searching for reviews...");
		List<Review> reviews = new ArrayList<Review>();
		URL url;
		BufferedReader in;
		String inputLine = "";
		String page = null;
		int counter = 0;
		boolean done = false;
		Matcher matchNextReview;
		try {
			for (int k = 0; k < (numberReviewsEachHotel/5) + 1; k++){
				if (k== 0){
					page = currentPage;
				}
				if (k == 1){
					page = nextPage;
				}
				if (page == null){
					break;
				}
				if (done == true){
					break;
				}
				Thread.sleep(sleepTime);
				url = new URL(urlBasic + page);
				in = new BufferedReader(new InputStreamReader(url.openStream(), "UTF-8"));
				String text = "";
				while ((inputLine = in.readLine()) != null){
					inputLine = inputLine.trim();
					matchNextReview = nextReviewPattern.matcher(inputLine);
					text = "";
					if (matchNextReview.find()){
						page = matchNextReview.group(1);
					}
					if (inputLine.contains("class=\"review extended provider0 inlineReviewUpdate") || inputLine.contains("<div class=\"col1of2\">")){
						if (inputLine.contains("<div class=\"col1of2\">")){
							text += inputLine;
						}
						inputLine = in.readLine();
						inputLine = inputLine.trim();
						if (inputLine.contains("<div class=\"col1of2\">") || inputLine.contains("<div class=\"member_info\">")){
							text += inputLine;
							while((inputLine = in.readLine()) != null){
								if (inputLine.contains("class=\"review extended provider0 inlineReviewUpdate")){
									break;
								}
								text += inputLine;
							}
							if (text.contains("<div class=\"googleTranslation reviewItem\">")){
								done = true;
								break;
							}
							Review rev = new Review();
							String headline = "";
							Matcher matchHeadline = headlinePattern.matcher(text);
							Matcher matchRating = ratingPattern.matcher(text);
							Matcher matchDate = datePattern.matcher(text);
							Matcher matchReviewText = reviewTextPattern.matcher(text);
							//Matcher matchRoomTip = roomTipPattern.matcher(text);
							Matcher matchProfile = profilePattern.matcher(text);
							Matcher matchLiked = likedPattern.matcher(text);
							Matcher matchDisliked = dislikedPattern.matcher(text);
							Matcher matchReviewHelpful = numberReviewHelpfulPattern.matcher(text);
							Matcher matchNumberUserReviews = numberUserReviewsPattern.matcher(text);
							Matcher matchNumberHelpfulUser = numberHelpfulUserPattern.matcher(text);
							Matcher matchUserName = userNamePattern.matcher(text);
							if (matchHeadline.find()){
								headline = matchHeadline.group(1);
							}
							if (matchRating.find()){
								rev.setScore(Double.parseDouble(matchRating.group(1))*2);
							}
							if (matchDate.find()){
								DateTime date = dtf.parseDateTime(matchDate.group(1));
								rev.setDate(date);
							}
							if (matchReviewText.find()){
								String temp = headline + ". " + matchReviewText.group(1);
								temp = temp.replaceAll("<.*?>", "");
								rev.setComment(temp);
							}
							//if (matchRoomTip.find()){
							////At the moment does not include room tip
							//}
							if (matchProfile.find()){
								rev.setProfile(matchProfile.group(1));
							}
							if (matchLiked.find()){
								rev.setCommentGood(matchLiked.group(1));
							}
							if (matchDisliked.find()){
								rev.setCommentBad(matchDisliked.group(1));
							}
							if (matchReviewHelpful.find()){
								String temp = matchReviewHelpful.group(1);
								temp = temp.replaceAll(",", "");
								rev.setHelpfulScore(Integer.parseInt(temp));
							}
							if (matchNumberUserReviews.find()){
								String temp = matchNumberUserReviews.group(1);
								temp = temp.replaceAll(",", "");
								if (temp.contains("helpful votes")){
									temp =temp.substring(0, 2);
								}
								rev.setReviewCount(Integer.parseInt(temp));
							}
							if (matchNumberHelpfulUser.find()){
								String temp = matchNumberHelpfulUser.group(1);
								temp = temp.replaceAll(",", "");
								if (temp.contains("1 helpful vote")){
									temp = "1";
								}
								rev.setTotalHelpfulScore(Integer.parseInt(temp));
							}
							if(matchUserName.find()){
								rev.setUsername(matchUserName.group(1));
							}
							if (rev.getUsername() == null){
								rev.setUsername("Anonymous");
							}
							reviews.add(rev);
							counter++;
							if (counter % 5 == 0){
								System.out.print(counter*(k+1) + "...");
								counter = 0;
							}
						}
					}
				}
			}} catch (MalformedURLException e) {
				e.printStackTrace();
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			} catch (SocketException e){
				return getReviews(currentPage, nextPage);
			} catch (IOException e) {
				e.printStackTrace();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		return reviews;
		}
	}
