package gatherer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.joda.time.DateTime;

import storers.Hotel;
import storers.Restaurant;
import storers.Review;
import storers.Service;
import storers.ServiceType;

/**
 * Class that reads the data set.
 *
 */
public class ReadDataSet {
	
	private String source = null;
	private String location = null;
	private Element root = null;
	private List<Service> services = null;

	public ReadDataSet() {
		services = new ArrayList<Service>();
	}
	public ReadDataSet(String source) {
		this();
		this.source = source;
	}
	public ReadDataSet(String source, String location) {
		this();
		this.source = source;
		this.location = location;
	}
	
	public void read() {
		this.read(location, true);
	}
	public void read(String location) {
		this.read(location, true);
	}
	public void read(boolean includeReviews) {
		this.read(location, includeReviews);
	}
	public void read(String location, boolean includeReviews) {
		String filename = this.source+"/"+location+".xml";
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(new File(filename));
			this.root = doc.getRootElement();
			makeServices(includeReviews);
		} catch(FileNotFoundException e) {
			System.err.println("File '"+filename+"' not found. Please choose a different source or location.");
		} catch(IOException e) {
			e.printStackTrace();
		} catch(JDOMException e) {
			e.printStackTrace();
		}
	}
	
	private void makeServices() {
		this.makeServices(true);
	}
	
	private void makeServices(boolean includeReviews) {
		List<Element> services = this.root.getChildren();
		for(Element serviceNode : services) {
			Service service = createService(serviceNode, includeReviews);
			this.services.add(service);
		}
	}
	
	/**
	 * Puts all the necessary information in a Service object
	 * @param serviceNode Service node
	 * @param includeReviews Shall each service include reviews also?
	 * @return Filled Service object
	 */
	private Service createService(Element serviceNode, boolean includeReviews) {
		ServiceType type = ServiceType.valueOf(serviceNode.getName().toUpperCase());
		Service service = null;
		if(type == ServiceType.HOTEL) {
			service = new Hotel(serviceNode.getAttributeValue("name"), serviceNode.getAttributeValue("address"), Double.parseDouble(serviceNode.getAttributeValue("stars")));
		} else if(type == ServiceType.RESTAURANT) {
			service = new Restaurant(serviceNode.getAttributeValue("name"), serviceNode.getAttributeValue("address"), Double.parseDouble(serviceNode.getAttributeValue("stars")));	
		}
		if(includeReviews) {
			List<Element> reviews = serviceNode.getChildren();
			for(Element reviewNode : reviews) {
				if(!reviewNode.getName().equals("review")) continue;
				Review review = createReview(reviewNode);
				service.addReview(review);
			}
		}
		if(service != null) {
			try {
				service.setLatitude(Double.parseDouble(serviceNode.getAttributeValue("latitude")));
				service.setLongitude(Double.parseDouble(serviceNode.getAttributeValue("longitude")));
			} catch(NullPointerException e) {
				System.err.println("Latitude and/or longitude not found");
			}
		}
		return service;
	}
	
	/**
	 * Puts all the reviews in a Review object
	 * @param reviewNode Element node
	 * @return A filled review object
	 */
	private Review createReview(Element reviewNode) {
		Review review = new Review();
		review.setUsername(reviewNode.getAttributeValue("user"));
		review.setReviewCount(Integer.parseInt(reviewNode.getAttributeValue("review_count")));
		review.setTotalHelpfulScore(Integer.parseInt(reviewNode.getAttributeValue("total_helpful")));
		review.setDate(new DateTime(reviewNode.getChildText("date")));
		review.setCountry(reviewNode.getChildText("country"));
		review.setProfile(reviewNode.getChildText("profile"));
		List<Element> texts = reviewNode.getChildren("text");
		for(Element textNode : texts) {
			review.setComment(textNode.getText(), textNode.getAttributeValue("sentiment"));
		}
		review.setHelpfulScore(Integer.parseInt(reviewNode.getChildText("number_helpful")));
		review.setScore(Double.parseDouble(reviewNode.getChildText("score")));
		
		return review;
	}
	
	
	//Getters and setters below
	public void setLocation(String location) {
		this.location = location;
	}
	public String getLocation() {
		return this.location;
	}
	public void setSource(String source) {
		this.source = source;
	}
	public String getSource() {
		return this.source;
	}
	public void setServices(List<Service> services) {
		this.services = services;
	}
	public List<Service> getServices() {
		return this.services;
	}
}
