package gatherer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.joda.time.DateTime;

import storers.Hotel;
import storers.Restaurant;
import storers.Review;
import storers.Service;
import storers.ServiceType;

/**
 * Class that reads the XML scores and stores them in memory
 *
 */
public class ReadScore {
	
	private String source = null;
	private String location = null;
	private Element root = null;
	private List<Service> services = null;
	
	public ReadScore() {
		services = new ArrayList<Service>();
	}
	public ReadScore(String source) {
		this();
		this.source = source;
	}
	
	public ReadScore(String source, String location) {
		this();
		this.source = source;
		this.location = location;
	}
	
	public void read() {
		this.read(location, true);
	}
	public void read(String location) {
		this.read(location, true);
	}
	public void read(boolean includeReviews) {
		this.read(location, includeReviews);
	}
	
	public void read(String location, boolean includeReviews) {
		String filename = this.source+"/score/"+location+".xml";
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(new File(filename));
			this.root = doc.getRootElement();
			makeServices(includeReviews);
		} catch(FileNotFoundException e) {
			System.err.println("File '"+filename+"' not found. Please choose a different source or location.");
		} catch(IOException e) {
			e.printStackTrace();
		} catch(JDOMException e) {
			e.printStackTrace();
		}
	}
	
	private void makeServices() {
		this.makeServices(true);
	}
	
	private void makeServices(boolean includeReviews) {
		List<Element> services = this.root.getChildren();
		for(Element serviceNode : services) {
			Service service = createService(serviceNode, includeReviews);
			if (service == null){
				System.out.println("null");
			}
			this.services.add(service);
		}
	}
	
	/**
	 * Puts all the necessary information in a Service object
	 * @param serviceNode Service node
	 * @param includeReviews Shall each service include reviews also?
	 * @return Filled Service object
	 */
	private Service createService(Element serviceNode, boolean includeReviews) {
		ServiceType type = ServiceType.valueOf(serviceNode.getName().toUpperCase());
		Service service = null;
		if(type == ServiceType.HOTEL) {
			service = new Hotel(serviceNode.getAttributeValue("name"), serviceNode.getAttributeValue("address"), 0.0);
		} else if(type == ServiceType.RESTAURANT) {
			service = new Restaurant(serviceNode.getAttributeValue("name"), serviceNode.getAttributeValue("address"), 0.0);	
		}
		if(includeReviews) {
			List<Element> reviews = serviceNode.getChildren();
			for(Element reviewNode : reviews) {
				Review review = createReview(reviewNode);
				service.addReview(review);
			}
		}
		if(service != null) {
			try {
				service.setLatitude(Double.parseDouble(serviceNode.getAttributeValue("latitude")));
				service.setLongitude(Double.parseDouble(serviceNode.getAttributeValue("longitude")));
			} catch(NullPointerException e) {
				System.err.println("Latitude and/or longitude not found");
			}
		}
		return service;
	}
	
	/**
	 * Puts all the reviews in a Review object
	 * @param reviewNode Element node
	 * @return A filled review object
	 */
	private Review createReview(Element reviewNode) {
		Review review = new Review();
		review.setUsername(null);
		review.setReviewCount(0);
		review.setTotalHelpfulScore(0);
		review.setDate(new DateTime(reviewNode.getChildText("date")));
		review.setCountry(null);
		review.setProfile(null);
		review.setComment(null);
		review.setHelpfulScore(0);
		review.setScore(Double.parseDouble(reviewNode.getChildText("score")));
		review.setNegScore(Double.parseDouble(reviewNode.getChildText("neg_score")));
		review.setPosScore(Double.parseDouble(reviewNode.getChildText("pos_score")));
		return review;
	}
	
	//Getters and setters below
	public void setLocation(String location) {
		this.location = location;
	}
	public String getLocation() {
		return this.location;
	}
	public void setSource(String source) {
		this.source = source;
	}
	public String getSource() {
		return this.source;
	}
	public void setServices(List<Service> services) {
		this.services = services;
	}
	public List<Service> getServices() {
		return this.services;
	}
	
	/**
	 * Check whether our sentiment scores is correct according to the actual scores given. 
	 */
	public void checkCorrectness(){
		System.out.println(getServices().size()+" services found");
		System.out.println();
		
		int correct = 0;
		int total = 0;
		double posScore = 0.0;
		double negScore = 0.0;
		double reviewScore = 0.0;
		List<Service> services = getServices();
		int number02 = 0;
		int number24 = 0;
		int number46 = 0;
		int number68 = 0;
		int number810 = 0;
		
		for (int i = 0; i < services.size(); i++){
			System.out.println(services.get(i).getName());
			System.out.println("Number of reviews: " + services.get(i).getReviews().size());
			System.out.println();
			posScore = 0.0;
			negScore = 0.0;
			for (int j = 0; j < services.get(i).getReviews().size(); j++){
				negScore = services.get(i).getReviews().get(j).getNegScore();
				posScore = services.get(i).getReviews().get(j).getPosScore();
				reviewScore = services.get(i).getReviews().get(j).getScore();
				System.out.println("Positive score: " + posScore);
				System.out.println("Negative score: " + negScore);
				System.out.println("Reviewer score: " + reviewScore);
				if (reviewScore > 0.0 && reviewScore <= 2.0){
					number02++;
				}
				if (reviewScore > 2.0 && reviewScore <= 4.0){
					number24++;
				}
				if (reviewScore > 4.0 && reviewScore <= 6.0){
					number46++;
				}
				if (reviewScore > 6.0 && reviewScore <= 8.0){
					number68++;
				}
				if (reviewScore > 8.0 && reviewScore <= 10.0){
					number810++;
				}
				
				//Can add different checks here if one wants to test different degrees of sentiment
//				if ((posScore > negScore) && reviewScore > 5.0){
//					correct++;
//					System.out.println("Correct!");
//				}
//				else if(negScore > posScore && services.get(i).getReviews().get(j).getScore() < 5.0){
//					correct++;
//					System.out.println("Correct!");
//				}
				if (((posScore - negScore) > 0.5) && (reviewScore >= 7.5)){
					correct++;
					System.out.println("Correct!");
				}
				else if (((posScore - negScore) > 0.0) && ((posScore - negScore) <= 0.5) && (reviewScore > 5.0) && (reviewScore < 7.5)){
					correct++;
					System.out.println("Correct!");
				}
				else if (((negScore - posScore) > 0.5) && (reviewScore <= 2.5)){
					correct++;
					System.out.println("Correct!");
				}
				else if (((negScore - posScore) > 0.0) && ((negScore - posScore) <= 0.5) && (reviewScore < 5.0) && (reviewScore > 2.5)){
					correct++;
					System.out.println("Correct!");
				}
				else if ((negScore == posScore) && (services.get(i).getReviews().get(j).getScore() == 5.0)){
				correct++;
				System.out.println("Correct!");
				}
				else {
					System.out.println("WRONG!");
				}
				System.out.println();
				total++;
			}
			System.out.println();
		}
		System.out.println("Number 0-2: " + number02);
		System.out.println("Number 2-4: " + number24);
		System.out.println("Number 4-6: " + number46);
		System.out.println("Number 6-8: " + number68);
		System.out.println("Number 8-10: " + number810);
		System.out.println("Number correct/total: " + correct + "/" + total + " = " + (correct/(double)total));
	}
}
