package output;

import gatherer.ReadScore;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.jdom.Element;
import org.jdom.output.XMLOutputter;
import org.joda.time.DateTime;

import storers.Review;
import storers.Service;

/**
 * Class that outputs the average score as xml in [source]/averages
 *
 */
public class AverageScoreOutputter {
	
	private Element locationNode = null;
	private List<Service> services = null;
	private ReadScore reader = null;
	
	public AverageScoreOutputter(String source, String location) {
		reader = new ReadScore(source, location);
		reader.read();
		services = reader.getServices();
		locationNode = new Element("location");
		locationNode.setAttribute("name", reader.getLocation());
		locationNode.setAttribute("last_updated", new DateTime().toString("yyyy-MM-dd"));
	}
	public AverageScoreOutputter() {
		
	}
	
	/**
	 * Creates a node for the service
	 * @param service Service, typically Hotel
	 * @return service node
	 */
	private Element createServiceNode(Service service) {
		Element serviceNode = new Element(service.getType().toString().toLowerCase());
		serviceNode.setAttribute("name", service.getName());
		serviceNode.setAttribute("address", service.getAddress());
		serviceNode.setAttribute("latitude", String.valueOf(service.getLatitude()));
		serviceNode.setAttribute("longitude", String.valueOf(service.getLongitude()));
		serviceNode.addContent(new Element("actual"));
		serviceNode.addContent(new Element("sentiment"));
		return serviceNode;
	}
	
	/**
	 * Calculate average sentiment score
	 * @param service Service
	 * @param serviceNode Service node
	 */
	private void calculateAverages(Service service, Element serviceNode) {
		Element totalNode = new Element("total");
		
		double score = 0;
		int counter = 0;
		for(Review review : service.getReviews()) {
			double pos = review.getPosScore();
			double neg = review.getNegScore();
			if(pos+neg==0) continue;
			score += pos/(pos+neg);
			counter += 1;
		}
		double averageScore = score/counter;
		Element el = new Element("score");
		el.setAttribute("reviews", String.valueOf(counter));
		el.setText(String.valueOf(averageScore));
		totalNode.addContent(el);
		serviceNode.getChild("sentiment").addContent(totalNode);
	}
	
	/**
	 * Calculates average score for each month. This is the sentiment value from SentiWordNet.
	 * @param service Service
	 * @param serviceNode Service node
	 */
	private void calculateMonthlyAverages(Service service, Element serviceNode) {
			
		Element monthlyNode = new Element("monthly");

		List<Review> reviews = sortReviewsOnDate(service.getReviews());
		DateTime firstDate = reviews.get(0).getDate();
		DateTime lastDate = reviews.get(reviews.size()-1).getDate();
		lastDate = new DateTime();

		Map<String,String> temp = new TreeMap<String,String>();
		double lastNonZeroValue = 0;

		for(int year = firstDate.getYear(); year <= lastDate.getYear(); year++) {
			for(int month = 1; month <= 12; month++) {
				if(year==lastDate.getYear() && month > lastDate.getMonthOfYear()) break;
				String key;
				if(month < 10) {
					key = String.valueOf(year)+"-0"+String.valueOf(month);
				} else {
					key = String.valueOf(year)+"-"+String.valueOf(month);
				}
				double score = 0;
				int counter = 0;
				for(Review review : reviews) {
					double pos = review.getPosScore();
					double neg = review.getNegScore();
					int y = review.getDate().getYear();
					int m = review.getDate().getMonthOfYear();
					if(pos+neg==0) continue;
					if(year==y && month==m) {
						score += pos/(pos+neg);
						counter += 1;
					} else {
						if(y>year) break;
					}
				}
				if(counter>0) {
					double totScore = score/counter;
					if(totScore>0) {
						lastNonZeroValue = totScore;
					} 
				}
				if(lastNonZeroValue > 0) {
					temp.put(key, lastNonZeroValue+"_"+counter);
					Element el = new Element("score");
					el.setAttribute("year", String.valueOf(year));
					el.setAttribute("month", String.valueOf(month));
					el.setAttribute("reviews", String.valueOf(counter));
					el.setText(String.valueOf(lastNonZeroValue));
					monthlyNode.addContent(el);
				}
			}
		}

		serviceNode.getChild("sentiment").addContent(monthlyNode);
	}
	
	/**
	 * Actual average scores based on the actual score given by reviewers.
	 * @param service Service
	 * @param serviceNode Service node
	 */
	private void actualAverages(Service service, Element serviceNode) {
		Element totalNode = new Element("total");
		
		double score = 0;
		int counter = 0;
		for(Review review : service.getReviews()) {
			score += review.getScore();
			counter += 1;
		}
		double averageScore = (score/counter)/10;
		Element el = new Element("score");
		el.setAttribute("reviews", String.valueOf(counter));
		el.setText(String.valueOf(averageScore));
		totalNode.addContent(el);
		serviceNode.getChild("actual").addContent(totalNode);
	}
	
	/**
	 * Calculates average score for each month. This is the actual value given by reviewers.
	 * @param service Service
	 * @param serviceNode Service node
	 */
	private void actualMonthlyAverages(Service service, Element serviceNode) {
		Element monthlyNode = new Element("monthly");

		List<Review> reviews = sortReviewsOnDate(service.getReviews());
		DateTime firstDate = reviews.get(0).getDate();
		DateTime lastDate = reviews.get(reviews.size()-1).getDate();
		lastDate = new DateTime();

		Map<String,String> temp = new TreeMap<String,String>();
		double lastNonZeroValue = 0;

		for(int year = firstDate.getYear(); year <= lastDate.getYear(); year++) {
			for(int month = 1; month <= 12; month++) {
				if(year==lastDate.getYear() && month > lastDate.getMonthOfYear()) break;
				String key;
				if(month < 10) {
					key = String.valueOf(year)+"-0"+String.valueOf(month);
				} else {
					key = String.valueOf(year)+"-"+String.valueOf(month);
				}
				double score = 0;
				int counter = 0;
				for(Review review : reviews) {
					int y = review.getDate().getYear();
					int m = review.getDate().getMonthOfYear();
					if(year==y && month==m) {
						score += review.getScore();
						counter += 1;
					} else {
						if(y>year) break;
					}
				}
				if(counter>0) {
					double totScore = score/counter;
					if(totScore>0) {
						lastNonZeroValue = totScore/10;
					} 
				}
				if(lastNonZeroValue > 0) {
					temp.put(key, lastNonZeroValue+"_"+counter);
					Element el = new Element("score");
					el.setAttribute("year", String.valueOf(year));
					el.setAttribute("month", String.valueOf(month));
					el.setAttribute("reviews", String.valueOf(counter));
					el.setText(String.valueOf(lastNonZeroValue));
					monthlyNode.addContent(el);
				}
			}
		}

		serviceNode.getChild("actual").addContent(monthlyNode);
	}
	
	private void addServiceNode(Element serviceNode) {
		locationNode.addContent(serviceNode);
	}
	
	/**
	 * Writes to file.
	 */
	public void write(){
		try {
			XMLOutputter fmt = new XMLOutputter();
			FileOutputStream fos = new FileOutputStream(reader.getSource()+"/averages/"+reader.getLocation()+".xml");
			fmt.output(locationNode, fos);
			System.out.println("File '"+reader.getSource()+"/averages/"+reader.getLocation()+".xml' updated");
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Sort reviews based on date.
	 * @param reviews Reviews
	 * @return Sorted reviews as a list
	 */
	private List<Review> sortReviewsOnDate(List<Review> reviews) {
		Collections.sort(reviews);
		return reviews;
	}
	
	public List<Service> getServices() {
		return this.services;
	}

	public void runAverageScoreOutputter(String source){
		File dir = new File(source+"/score/");
		for(String file : dir.list()) {
			AverageScoreOutputter asc = new AverageScoreOutputter(source, file.split("\\.")[0]);
			for(Service service : asc.getServices()) {
				Element serviceNode = asc.createServiceNode(service);
				asc.calculateMonthlyAverages(service, serviceNode);
				asc.calculateAverages(service, serviceNode);
				asc.actualMonthlyAverages(service, serviceNode);
				asc.actualAverages(service, serviceNode);
				asc.addServiceNode(serviceNode);
			}
			if(asc.getServices() != null) {
				asc.write();
			}
		}
	}
}
