package output;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.IllegalDataException;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;
import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import storers.Hotel;
import storers.Review;
import storers.Service;

/**
 * Class that outputs the data sets in [source] folder. 
 *
 */
public class DataSetOutputter {

	private String source = null;
	private String location = null;
	private boolean fileExists = false;
	private Element locationNode = null;
	private Element serviceNode = null;
	private DateTime lastUpdated = null;
	private List<Element> locationServices = null;

	private DateTimeFormatter dtf = DateTimeFormat.forPattern("yyyy-MM-dd").withZone(DateTimeZone.UTC);

	public DataSetOutputter(String source, String location) {
		this.source = source; 
		this.location = location;
		readXML();
	}

	/**
	 * Read XML if the file exist from before.
	 */
	public void readXML() {
		String filename = source+"/"+location+".xml";
		try {
			// Build the document with SAX and Xerces, no validation
			SAXBuilder builder = new SAXBuilder();
			// Create the document
			Document doc = builder.build(new File(filename));
			// Output the document, use standard formatter
			this.locationNode = doc.getRootElement();
			fileExists = true;
		} catch(IOException e) {
			fileExists = false;
		} catch(JDOMException e) {
			e.printStackTrace();
		}
	}
	
	public boolean serviceExists(String serviceName) {
		Service service = new Hotel(serviceName, null, 0);
		return serviceExists(service);
	}

	/**
	 * Checks if a service exist from before.
	 * @param service Service
	 * @return True if it exist, false if not
	 */
	public boolean serviceExists(Service service) {
		locationExists();
		if(locationServices == null) {
			return false;
		}
		for(Element el : locationServices) {
			if(el.getAttributeValue("name").equals(service.getName())) {
				serviceNode = el;
				return true;
			}
		}
		return false;
	}

	/**
	 * Checks whether a location exist from before
	 * @return True if it exist, false if not
	 */
	public boolean locationExists() {
		if(fileExists) {
			locationServices = this.locationNode.getChildren();
			lastUpdated = dtf.parseDateTime(this.locationNode.getAttributeValue("last_updated"));
			return true;
		}
		return false;
	}
	
	public void output(Service service) {
		List<Service> temp = new ArrayList<Service>();
		temp.add(service);
		this.output(temp);
	}

	/**
	 * Builds the XML tree and writes it to file afterwards.
	 * @param services List of services
	 */
	public void output(List<Service> services) { 
		boolean locationExists = locationExists();
		boolean newService = true;
		if(!locationExists) {
			locationNode = new Element("location");
			locationNode.setAttribute("name", location);
			lastUpdated = new DateTime(0);
			locationNode.setAttribute("last_updated", lastUpdated.toString("yyyy-MM-dd"));
		}
		for(Service service : services) {
			if(!serviceExists(service)) {
				serviceNode = new Element(service.getType().toString().toLowerCase());
				serviceNode.setAttribute("name", service.getName());
				serviceNode.setAttribute("address", service.getAddress());
				serviceNode.setAttribute("stars", String.valueOf(service.getStars()));
				serviceNode.setAttribute("latitude", String.valueOf(service.getLatitude()));
				serviceNode.setAttribute("longitude", String.valueOf(service.getLongitude()));
				if(service.getFacilities() != null) {
					Element facilityNode = new Element("facilities");
					for(Entry<String,String> entry : service.getFacilities().entrySet()) {
						facilityNode.addContent(new Element(entry.getKey()).setText(entry.getValue()));
					}
					serviceNode.addContent(facilityNode);
				}
				newService = true;
			} else {
				newService = false;
			}
			for(Review review : service.getReviews()) {
				if(lastUpdated.isBefore(review.getDate()) || newService) {
					Element reviewNode = new Element("review");
					reviewNode.setAttribute("user", review.getUsername());
					reviewNode.setAttribute("review_count", String.valueOf(review.getReviewCount()));
					reviewNode.setAttribute("total_helpful", String.valueOf(review.getTotalHelpfulScore()));
					reviewNode.addContent(new Element("date").setText(review.getDate().toString("yyyy-MM-dd")));
					reviewNode.addContent(new Element("country").setText(review.getCountry()));
					reviewNode.addContent(new Element("profile").setText(review.getProfile()));
					try {
						if(review.getComment()!=null) {
							reviewNode.addContent(new Element("text").setText(review.getComment()).setAttribute("sentiment", ""));
						}
						if(review.getCommentGood()!=null) {
							reviewNode.addContent(new Element("text").setText(review.getCommentGood()).setAttribute("sentiment", "good"));
						}
						if(review.getCommentBad()!=null) {
							reviewNode.addContent(new Element("text").setText(review.getCommentBad()).setAttribute("sentiment", "bad"));
						}
					} catch(IllegalDataException e) {
						System.err.println(e.getMessage());
					}
					reviewNode.addContent(new Element("number_helpful").setText(String.valueOf(review.getHelpfulScore())));
					reviewNode.addContent(new Element("score").setText(String.valueOf(review.getScore())));

					serviceNode.addContent(reviewNode);
				}
			}
			if(newService) {
				locationNode.addContent(serviceNode);
			}
		}
		locationNode.setAttribute("last_updated", new DateTime().toString("yyyy-MM-dd"));
		
		write();
	} 
	
	/**
	 * Write to file.
	 */
	public void write() {
		try {
			XMLOutputter fmt = new XMLOutputter();
			FileOutputStream fos = new FileOutputStream(source+"/"+location+".xml");
			fmt.output(locationNode, fos);
			System.out.println("File '"+source+"/"+location+".xml' updated");
			fileExists = true;
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	public void forceUpdate() {
		this.lastUpdated = new DateTime(0);
	}
	
	public void setLastUpdated(DateTime date) {
		this.lastUpdated = date;
	}
	
	public DateTime getLastUpdated() {
		return this.lastUpdated;
	}
	public Element getServiceNode() {
		return this.serviceNode;
	}


	public static void main(String[] args) {
		//		new OutputXML().output(null, "booking.com");
	}
}
