package output;

import gatherer.ReadDataSet;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jdom.Element;
import org.jdom.output.XMLOutputter;
import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import processing.PosTagger;
import processing.SentiWordNet;
import processing.WordNet;
import storers.Review;
import storers.SentiWord;
import storers.Service;


/**
 * Class for outputting the scores in [source]/scores as XML files
 *
 */
public class ScoreOutputter {
	
	private String source = null;
	private String location = null;
	private boolean fileExists = false;
	private Element locationNode = null;
	private Element serviceNode = null;
	private DateTime lastUpdated = null;
	private List<Element> locationServices = null;
	private String scoreFolder = "score";
	private boolean lesk;
	
	private DateTimeFormatter dtf = DateTimeFormat.forPattern("yyyy-MM-dd").withZone(DateTimeZone.UTC);

	public ScoreOutputter() {
		lesk = false;
		scoreFolder = "score";
	}
	public ScoreOutputter(boolean lesk) {
		this.lesk = lesk;
		if(lesk) {
			scoreFolder = "scoreLesk";
		} else {
			scoreFolder = "score";
		}
	}

	/**
	 * Analyzes the sentiment score for each word.
	 * @param reader reader of the dataset file with all the service information
	 */
	public void scoreAnalyzer(ReadDataSet reader){
		System.out.println("Analyzing the sentiment score for each word...");
		SentiWordNet swn = new SentiWordNet();
		swn.addSentiWords();
		HashMap<String, SentiWord> sws = swn.getSentiWords();

		WordNet wn = new WordNet();
		wn.openDictionary();
		double totalPos = 0.0;
		double totalNeg = 0.0;
		List<Service> services = reader.getServices();
		
		for (int i = 0; i < services.size(); i++){
			System.out.println(i);
			for (int j = 0; j < services.get(i).getReviews().size(); j++){
				HashMap<String, Integer> map = services.get(i).getReviews().get(j).getTaggedWords();
				totalPos = 0.0;
				totalNeg = 0.0;
				for (Map.Entry<String, Integer> entry : map.entrySet()){
					String word = entry.getKey().split("_")[0];
					String wordType = entry.getKey().split("_")[1];
					
					//One can either call simplifiedLest or just pick the most common ID through getID
					String id;
					if(lesk) {
						id = wn.simplifiedLesk(word, wordType, map);
					} else {
						id = wn.getID(word, wordType);
					}
					
					if (id == null){
						continue;
					}
					SentiWord sw = sws.get(id+wordType);
					totalPos += sw.getPosValue();
					totalNeg += sw.getNegValue();
				}
				services.get(i).getReviews().get(j).setNegScore(totalNeg);
				services.get(i).getReviews().get(j).setPosScore(totalPos);
			}
		}
	}
	
	/**
	 * Writes it to file as XML
	 * @param reader Reader
	 */
	public void writeXML(ReadDataSet reader){
		boolean locationExists = locationExists();
		boolean newService = true;
		
		if(!locationExists) {
			locationNode = new Element("location");
			locationNode.setAttribute("name", reader.getLocation());
			lastUpdated = new DateTime(0);
			locationNode.setAttribute("last_updated", lastUpdated.toString("yyyy-MM-dd"));
		}
		List<Service> services = reader.getServices();
		for (int i = 0; i < services.size(); i++){
			if(!(serviceExists(services.get(i))) && services.get(i).getReviews().size() > 0){
				serviceNode = new Element(services.get(i).getType().toString().toLowerCase());
				serviceNode.setAttribute("name", services.get(i).getName());
				serviceNode.setAttribute("address", services.get(i).getAddress());
				serviceNode.setAttribute("latitude", String.valueOf(services.get(i).getLatitude()));
				serviceNode.setAttribute("longitude", String.valueOf(services.get(i).getLongitude()));
				newService = true;
			}
			else {
				newService = false;
			}
			for(Review review : services.get(i).getReviews()) {
				if(lastUpdated.isBefore(review.getDate()) || newService) {
					Element reviewNode = new Element("review");
					reviewNode.addContent(new Element("date").setText(review.getDate().toString("yyyy-MM-dd")));
					reviewNode.addContent(new Element("score").setText(String.valueOf(review.getScore())));
					reviewNode.addContent(new Element("pos_score").setText(String.valueOf(review.getPosScore())));
					reviewNode.addContent(new Element("neg_score").setText(String.valueOf(review.getNegScore())));
					
					serviceNode.addContent(reviewNode);
				}
			}
			if(newService) {
				locationNode.addContent(serviceNode);
			}
		}
		locationNode.setAttribute("last_updated", new DateTime().toString("yyyy-MM-dd"));
		
		try {
			XMLOutputter fmt = new XMLOutputter();
			FileOutputStream fos = new FileOutputStream(reader.getSource()+"/"+scoreFolder+"/"+reader.getLocation()+".xml");
			fmt.output(locationNode, fos);
			System.out.println("File '"+reader.getSource()+"/"+scoreFolder+"/"+reader.getLocation()+".xml' updated");
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Checks wheter the service exist from before or not
	 * @param service Service
	 * @return True if it exist, false if not
	 */
	public boolean serviceExists(Service service) {
		locationExists();
		if(locationServices == null) {
			return false;
		}
		for(Element el : locationServices) {
			if(el.getAttributeValue("name").equals(service.getName())) {
				serviceNode = el;
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Checks whether the location exist from before
	 * @return True if it does, false if not
	 */
	public boolean locationExists() {
		if(fileExists) {
			locationServices = this.locationNode.getChildren();
			lastUpdated = dtf.parseDateTime(this.locationNode.getAttributeValue("last_updated"));
			return true;
		}
		return false;
	}
	
	public void runScoreOutputter(String source, String location, String lesk){
		PosTagger pos = new PosTagger();
		ReadDataSet reader = pos.tagReviews(source, location);
		ScoreOutputter sop;
		if (lesk.equals("lesk")){
			sop = new ScoreOutputter(true); //true for Lesk, false otherwise
		}
		else {
			sop = new ScoreOutputter(false); //true for Lesk, false otherwise
		}
		sop.scoreAnalyzer(reader);
		sop.writeXML(reader);
	}
}
