package processing;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import storers.SentiWord;

import crawler.ParserInterface;

import edu.mit.jwi.Dictionary;
import edu.mit.jwi.IDictionary;
import edu.mit.jwi.item.IIndexWord;
import edu.mit.jwi.item.IWord;
import edu.mit.jwi.item.IWordID;
import edu.mit.jwi.item.POS;
import edu.mit.jwi.morph.WordnetStemmer;
import gatherer.ReadDataSet;

/**
 * Class for the WordNet implementation.
 *
 */
public class WordNet {
	private IDictionary dict;
	private WordnetStemmer wns;

	public WordNet(){
		dict = null;
		wns = null;
	}

	/**
	 * Opens up the WordNet dictionary
	 */
	public void openDictionary (){
		// Construct the URL to the Wordnet dictionary directory
		String wnhome = "";
		String term = System.getenv("TERM");
		String os = System.getenv("OS");

		// Default path on Linux
		if (term != null && term.equals("linux")){
			wnhome = "/usr/local/WordNet-3.0";
		}

		// Default path on Windows
		else if (term == null && os.equals("Windows_NT")){
			wnhome = "C:\\Program Files (x86)\\WordNet\\2.1";
		}
		else {
			System.err.println("Fant ikke riktig sti for WordNet dict");
			System.exit(1);
		}

		String path = wnhome + File.separator + "dict";
		URL url;
		try {
			url = new URL ("file", null, path);

			// Construct the dictionary object and open it
			dict = new Dictionary(url);
			dict.open();
			wns = new WordnetStemmer(dict);
		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Gets the WordNet ID
	 * @param word Word
	 * @param wordType Type of the word, can be adjective, verb, noun, adverb
	 * @return WordNet ID
	 */
	public String getID(String word, String wordType){
		POS p = null;
		if (wordType.equals("a")){
			p = POS.ADJECTIVE;
		}
		if (wordType.equals("v")){
			p = POS.VERB;
		}
		if (wordType.equals("n")){
			p = POS.NOUN;
		}
		if (wordType.equals("r")){
			p = POS.ADVERB;
		}
		if (p == null){
			return null;
		}
		List<String> stems;
		try {
			stems = wns.findStems(word, p);
		}
		catch(IllegalArgumentException e){
			return null;
		}
		//System.out.println();
		//System.out.println(stems);
		if (stems.size() == 0 || stems.get(0).contains("http:\\")){
			return null;
		}
		IIndexWord idxWord;
		try {
			idxWord = dict.getIndexWord(stems.get(0), p);
		}
		catch(NullPointerException e){
			System.out.println(stems.get(0));
			System.out.println("NULLPOINTER");
			return null;
		}
		if (idxWord == null){
			if (stems.size() > 1){
				idxWord = dict.getIndexWord(stems.get(1), p);
			}
		}
		if (idxWord == null){
			return null;
		}
		IWordID wordID = idxWord.getWordIDs().get(0);
		String id = wordID.toString().split("-")[1];
		return id;
	}

	/**
	 * Implementation of a simplified version of the Lesk algorithm
	 * @param word Word
	 * @param wordType Type of the word
	 * @param map HashMap over each word and the number of occurences
	 * @return Word ID
	 */
	public String simplifiedLesk(String word, String wordType, HashMap<String, Integer> map){
		String id = null;
		POS p = null;
		if (wordType.equals("a")){
			p = POS.ADJECTIVE;
		}
		if (wordType.equals("v")){
			p = POS.VERB;
		}
		if (wordType.equals("n")){
			p = POS.NOUN;
		}
		if (wordType.equals("r")){
			p = POS.ADVERB;
		}
		if (p == null){
			return null;
		}
		List<String> stems;
		try {
			stems = wns.findStems(word, p);
		}
		catch(IllegalArgumentException e){
			return null;
		}
		if (stems.size() == 0 || stems.get(0).contains("http:\\")){
			return null;
		}
		IIndexWord idxWord;
		try {
			idxWord = dict.getIndexWord(stems.get(0), p);
		}
		catch(NullPointerException e){
			System.out.println(stems.get(0));
			System.out.println("NULLPOINTER");
			return null;
		}
		if (idxWord == null){
			if (stems.size() > 1){
				idxWord = dict.getIndexWord(stems.get(1), p);
			}
		}
		if (idxWord == null){
			return null;
		}
		IWordID wordID = idxWord.getWordIDs().get(0);
		id = wordID.toString().split("-")[1];
		IWord iWord = dict.getWord(wordID);
		String gloss1 = iWord.getSynset().getGloss();
		gloss1 = gloss1.replaceAll("[^A-Za-z0-9\\\" \"]", "");
		String[] glossWords1 = gloss1.split(" ");
		int bestMatches = 0;
		for (int k = 0; k < glossWords1.length; k++){
			for (Map.Entry<String, Integer> entry : map.entrySet()){
				String wordTemp = entry.getKey().split("_")[0];
				if (wordTemp.equals(glossWords1[k])){
					bestMatches++;
					break;
				}
			}
		}
		for (int i = 0; i < iWord.getRelatedWords().size(); i++){
			int numberMatches = 0;
			iWord = dict.getWord(iWord.getRelatedWords().get(i));
			if (iWord.getPOS().equals(p)){
				//	System.out.println("Gloss: " + word.getSynset().getGloss());
				String gloss2 = iWord.getSynset().getGloss();
				gloss2 = gloss2.replaceAll("[^A-Za-z0-9\\\" \"]", "");
				System.out.println(gloss2);
				String[] glossWords2 = gloss2.split(" ");
				for (int j = 0; j < glossWords2.length; j++){
					for (Map.Entry<String, Integer> entry : map.entrySet()){
						String wordTemp = entry.getKey().split("_")[0];
						if (wordTemp.equals(glossWords2[j])){
							numberMatches++;
							break;
						}
					}
				}
				if (numberMatches > bestMatches){
					bestMatches = numberMatches;
					id = iWord.getID().toString().split("-")[1];
				}
			}
		}
		return id;

	}
	public static void main(String[] args){
		WordNet wn = new WordNet();
		wn.openDictionary();
		IIndexWord idxWord = wn.dict.getIndexWord("cone", POS.NOUN);
		IWordID wordID = idxWord.getWordIDs().get(0);
		IWord word = wn.dict.getWord(wordID);
		String gloss1 = word.getSynset().getGloss();
		gloss1 = gloss1.replaceAll("[^A-Za-z0-9\\\" \"]", "");
		String[] glossWords1 = gloss1.split(" ");
		for (int k = 0; k < glossWords1.length; k++){

		}
		System.out.println("Gloss: " + gloss1);
		for (int i = 0; i < word.getRelatedWords().size(); i++){
			word = wn.dict.getWord(word.getRelatedWords().get(i));
			if (word.getPOS().equals(POS.NOUN)){
				//				System.out.println("Gloss: " + word.getSynset().getGloss());
				String gloss2 = word.getSynset().getGloss();
				gloss2 = gloss2.replaceAll("[^A-Za-z0-9\\\" \"]", "");
				System.out.println(gloss2);
				String[] glossWords2 = gloss2.split(" ");
				for (int j = 0; j < glossWords2.length; j++){

				}
			}
			System.out.println(word.getID().toString().split("-")[1]);
		}
	}
}
